@@ -2,7 +2,7 @@ package Net::Dropbear::SSHd;
 
 use strict;
 use v5.8;
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 use Child;
 
@@ -62,6 +62,8 @@ has comm => (
   isa => GlobRef,
 );
 
+my $tell_parent;
+
 sub is_running
 {
   my $self = shift;
@@ -86,9 +88,16 @@ sub run
     my $parent = shift;
     $0 .= " [Net::Dropbear Child]";
 
-    $parent_comm->close;
     $self->_set_comm($child_comm);
 
+    $tell_parent = sub
+    {
+      # Tell the parent we're ready by writing to the socket we normally read
+      # from.
+      $parent_comm->print("\n");
+      $parent_comm->close;
+    };
+
     require Net::Dropbear::XS;
 
     Net::Dropbear::XS->setup_svr_opts($self);
@@ -103,9 +112,12 @@ sub run
   });
 
   $self->_set_child($child->start);
-  $child_comm->close;
   $self->_set_comm($parent_comm);
 
+  # Wait for child to come up by reading from the socket we normally close
+  $child_comm->getline;
+  $child_comm->close;
+
   return;
 }
 
@@ -132,6 +144,12 @@ sub auto_hook
   my $self = shift;
   my $hook = shift;
 
+  if ($hook eq 'on_start' && ref $tell_parent eq 'CODE')
+  {
+    $tell_parent->();
+    undef $tell_parent;
+  }
+
   if (exists $self->hooks->{$hook})
   {
     return $self->hooks->{$hook}->(@_);
@@ -451,7 +469,7 @@ B<HOOK_FAILURE> - The command session will behave as though the command had vail
 
 =head2 run
 
-Call C<$sshd->run> to start Dropbear.
+Call C<$sshd-E<gt>run> to start Dropbear.
 
 =head2 child
 
